//	yf Net differencer
//		2005.09.29 - 2005.10.08 Yoshiaki.Ueda
#include "wx/wx.h"
#include "yfNetDifferencer.h"
#include "yfNetList.h"
#include "yfNetListReaders1.h"
#include "yfNetMode.h"

//yfNetDifferencer::Node//=====================================================

	//Initialize
//yfNetDifferencer::Diff::Diff(
//		unsigned differNode, yfNetList::Net* m_file1Net, yfNetList::Net* m_file2Net )
//	: m_status((differNode==0)? SAME:LIKE )
//	, m_differNode(differNode),	m_file1Net(m_file1Net), m_file2Net(m_file2Net)
//{
//}

yfNetDifferencer::Diff::Diff(
		STATUS status, yfNetList::Net* m_file1Net, yfNetList::Net* m_file2Net )
	: m_status(status), m_differNode(UINT_MAX), m_file1Net(m_file1Net), m_file2Net(m_file2Net)
{
}

//yfNetDifferencer//===========================================================
	//Execute
bool yfNetDifferencer::Execute(
		const wxString& file1Name, unsigned file1Mode,
		const wxString& file2Name, unsigned file2Mode )
{
	//Read
	yfNetListReader* reader = yfNetMode::GetReader(file1Mode);
	if(!reader) return false;
	reader->Read(file1Name, m_net1List);
	delete reader;
	m_net1List.Sort();

	reader = yfNetMode::GetReader(file2Mode);
	if(!reader) return false;
	reader->Read(file2Name, m_net2List);
	delete reader;
	m_net2List.Sort();

	//scan file 1
	yfNetList::List::iterator net;
	for(net=m_net1List.m_list.begin(); net!=m_net1List.m_list.end(); net++){
		yfNetList::PICKUP_STATUS status;
		yfNetList::Net* otherNet = m_net2List.Pickup(&*net, status);
		if(otherNet){
			//some or like
			yfNetDifferencer::Diff diff(
				status==yfNetList::SAME? Diff::SAME : Diff::LIKE, &*net, otherNet);
			m_list.push_back(diff);
		}else{
			//file1 only
			yfNetDifferencer::Diff diff(Diff::FILE1_ONLY, &*net, NULL);
			m_list.push_back(diff);
		}
	}

	//scan file 2
	for(net=m_net2List.m_list.begin(); net!=m_net2List.m_list.end(); net++){
		//file2 only
		if(!net->m_pickup){
			yfNetDifferencer::Diff diff(Diff::FILE2_ONLY, NULL, &*net);
			m_list.push_back(diff);
		}
	}
	return true;
}

	//text
void yfNetDifferencer::GetText(wxTextCtrl& text) const
{
	List::const_iterator diff;
	//same
	text.AppendText(wxT("[[Match]]\n"));
	for(diff=m_list.begin(); diff!=m_list.end(); diff++){
		if(diff->m_status==Diff::SAME){
			diff->m_file1Net->GetText(text);
			text.AppendText(wxT(" == "));
			diff->m_file2Net->GetText(text);
			text.AppendText(wxT('\n'));
		}
	}
	//like
	text.AppendText(wxT("\n[[Not Match]]\n"));
	for(diff=m_list.begin(); diff!=m_list.end(); diff++){
		if(diff->m_status==Diff::LIKE){
			diff->m_file1Net->GetText(text);
			text.AppendText(wxT(" != "));
			diff->m_file2Net->GetText(text);
			text.AppendText(wxT('\n'));
		}
	}
	//file 1 only
	text.AppendText(wxT("\n<<File 1 only>>\n"));
	for(diff=m_list.begin(); diff!=m_list.end(); diff++){
		if(diff->m_status==Diff::FILE1_ONLY){
			diff->m_file1Net->GetText(text);
			text.AppendText(wxT('\n'));
		}
	}
	//file 2 only
	text.AppendText(wxT("\n<<File 2 only>>\n"));
	for(diff=m_list.begin(); diff!=m_list.end(); diff++){
		if(diff->m_status==Diff::FILE2_ONLY){
			diff->m_file2Net->GetText(text);
			text.AppendText(wxT('\n'));
		}
	}
}

const wxString yfNetDifferencer::GetText() const
{
	wxString text;
	List::const_iterator diff;
	//same
	text += wxT("[[Match]]\n");
	for(diff=m_list.begin(); diff!=m_list.end(); diff++){
		if(diff->m_status==Diff::SAME){
			text += diff->m_file1Net->GetText();
			text += wxT(" == ");
			text += diff->m_file2Net->GetText();
			text += wxT('\n');
		}
	}
	//like
	text += wxT("\n[[Not Match]]\n");
	for(diff=m_list.begin(); diff!=m_list.end(); diff++){
		if(diff->m_status==Diff::LIKE){
			text += diff->m_file1Net->GetText();
			text += wxT(" != ");
			text += diff->m_file2Net->GetText();
			text += wxT('\n');
		}
	}
	//file 1 only
	text += wxT("\n<<File 1 only>>\n");
	for(diff=m_list.begin(); diff!=m_list.end(); diff++){
		if(diff->m_status==Diff::FILE1_ONLY){
			text += diff->m_file1Net->GetText();
			text += wxT('\n');
		}
	}
	//file 2 only
	text += wxT("\n<<File 2 only>>\n");
	for(diff=m_list.begin(); diff!=m_list.end(); diff++){
		if(diff->m_status==Diff::FILE2_ONLY){
			text += diff->m_file2Net->GetText();
			text += wxT('\n');
		}
	}
	return text;
}
